@@ -19,7 +19,6 @@ feature_labels = [
     'subleading_energy',
     'subleading_mt',
     'subleading_rt',
-    'subleading_rtnew',
     'subleading_msd',
     'subleading_axismajor',
     'subleading_axisminor',
@@ -32,7 +31,6 @@ feature_labels = [
     'leading_energy',
     'leading_mt',
     'leading_rt',
-    'leading_rtnew',
     'leading_msd',
     'leading_axismajor',
     'leading_axisminor',
@@ -40,6 +38,13 @@ feature_labels = [
     'leading_ecfN2b2',
     'leading_girth',
     'leading_ptd',
+    # More RT values
+    'leading_rtnew',
+    'leading_rt1',
+    'subleading_rtnew',
+    'subleading_rt1',
+    'rt2',
+    # Calculated features
     'subleading_tau21',
     'leading_tau21',
     'deltaeta',
@@ -50,6 +55,11 @@ def to_feature_array(arrays):
     """
     Transforms a dict-like arrays object to a flat numpy array for ML purposes
     """
+    # Add some RT branches
+    calculate_other_rts(arrays)
+    calculate_rt2(arrays)
+    calculate_other_rts(arrays, b'JetsAK15_subleading')
+    calculate_rt2(arrays, b'JetsAK15_subleading')
     # Simply direct variables from the arrays object
     plain_features = [
         b'MET',
@@ -58,7 +68,6 @@ def to_feature_array(arrays):
         b'JetsAK15_subleading.fCoordinates.fE',
         b'JetsAK15_subleading_MT',
         b'JetsAK15_subleading_RT',
-        b'JetsAK15_subleading_RTnew',
         b'JetsAK15_subleading_softDropMass',
         b'JetsAK15_subleading_axismajor',
         b'JetsAK15_subleading_axisminor',
@@ -71,7 +80,6 @@ def to_feature_array(arrays):
         b'JetsAK15_leading.fCoordinates.fE',
         b'JetsAK15_leading_MT',
         b'JetsAK15_leading_RT',
-        b'JetsAK15_leading_RTnew',
         b'JetsAK15_leading_softDropMass',
         b'JetsAK15_leading_axismajor',
         b'JetsAK15_leading_axisminor',
@@ -79,6 +87,12 @@ def to_feature_array(arrays):
         b'JetsAK15_leading_ecfN2b2',
         b'JetsAK15_leading_girth',
         b'JetsAK15_leading_ptD',
+        # 
+        b'JetsAK15_leading_RTnew',
+        b'JetsAK15_leading_RT1',
+        b'JetsAK15_subleading_RTnew',
+        b'JetsAK15_subleading_RT1',
+        b'JetsAK15_RT2'
         ]
     # Quantities that have to be calculated from branches
     calculated_features = [
@@ -92,8 +106,9 @@ def to_feature_array(arrays):
     plain = [ arrays[b].flatten() for b in plain_features ]
     for fn in calculated_features:
         plain.append(fn(arrays).flatten())
-    plain = np.array(plain)
-    return plain.T
+    plain = np.array(plain).T
+    assert plain.shape[1] == len(plain_features) + len(calculated_features)
+    return plain
 
 # ------------------------
 # Event level
@@ -188,8 +203,11 @@ def count_triggers(trigger_decisions, triggers):
     return n_pass #, n_total
 
 def filter_zerojet_events(arrays, inplace=True):
-    jets = get_jets(arrays, b'JetsAK15')
-    passes = (jets.counts >= 1)
+    return require_minimum_njets(arrays, 1, inplace)
+
+def require_minimum_njets(arrays, njets=1, inplace=True):
+    jets = get_jets(arrays, b'JetsAK15', has_subjets=False)
+    passes = (jets.counts >= njets)
     return select(arrays, passes, inplace)
 
 def is_nested(arrays):
@@ -251,7 +269,18 @@ class Jets(object):
         'ptD',
         ]
 
-    def __init__(self, name, arrays=None, attributes=None, has_subjets=True, has_substructure=True):
+    @classmethod
+    def from_ptetaphie(cls, name, pt, eta, phi, energy):
+        instance = cls(name, attributes=[], has_subjets=False, has_substructure=False)
+        instance.pt = pt
+        instance.eta = eta
+        instance.phi = phi
+        instance.energy = energy
+        instance.mass2 = instance._mag2()
+        instance.mass = instance._mass()
+        return instance
+
+    def __init__(self, name, arrays=None, attributes=None, has_subjets=True, has_preprocessed_subjets=False, has_substructure=True):
         self.has_subjets = has_subjets
         self.has_substructure = has_substructure
         self.name = name
@@ -269,17 +298,24 @@ class Jets(object):
             self.softdropmass = arrays[add_to_bytestring(self.name, '_softDropMass')]
             
             if self.has_subjets:
-                for offsets_branch in [ '_subjetsOffsets', '_subjetsCounts' ]:
-                    if add_to_bytestring(self.name, offsets_branch) in arrays.keys():
-                        break
+                if has_preprocessed_subjets:                
+                    self.n_subjets = arrays[add_to_bytestring(self.name, '_subjetsCounts')]
+                    self.subjet_pt = arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fPt')]
+                    self.subjet_eta = arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fEta')]
+                    self.subjet_phi = arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fPhi')]
+                    self.subjet_energy = arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fE')]
                 else:
-                    raise Exception('Could not determine any subjets offset branch in arrays')
-                self.offsets_branch = offsets_branch
-                self.n_subjets = arrays[add_to_bytestring(self.name, offsets_branch)]
-                self.subjet_pt = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fPt')])
-                self.subjet_eta = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fEta')])
-                self.subjet_phi = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fPhi')])
-                self.subjet_energy = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fE')])
+                    for offsets_branch in [ '_subjetsOffsets', '_subjetsCounts' ]:
+                        if add_to_bytestring(self.name, offsets_branch) in arrays.keys():
+                            break
+                    else:
+                        raise Exception('Could not determine any subjets offset branch in arrays')
+                    self.offsets_branch = offsets_branch
+                    self.n_subjets = arrays[add_to_bytestring(self.name, offsets_branch)]
+                    self.subjet_pt = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fPt')])
+                    self.subjet_eta = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fEta')])
+                    self.subjet_phi = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fPhi')])
+                    self.subjet_energy = self._doublejagged_from_nsubjet(arrays[add_to_bytestring(self.name, '_subjets.fCoordinates.fE')])
 
             if self.has_substructure:
                 for attr in self.subtructure_branches:
@@ -299,7 +335,7 @@ class Jets(object):
         mag2[mag2 < 0.] = 0.
         return mag2
         # return np.sqrt(mag2, out=np.zeros_like(mag2), where=mag2>0.)
-        
+    
     def _doublejagged_from_nsubjet(self, array):
         """
         Returns a 'nested' JaggedArray based on n_subjets
@@ -437,11 +473,25 @@ def calculate_mt(arrays, jets_branch=b'JetsAK15_leading'):
 
     mt = np.sqrt( (jetse + mete)**2 - (jetsx + metx)**2 - (jetsy + mety)**2 )
     rt = arrays[b'MET'] / mt
-    rtnew = arrays[b'MET'] / np.sqrt(mt**2 + jets.pt**2)
     arrays[add_to_bytestring(jets_branch, '_MT')] = mt
     arrays[add_to_bytestring(jets_branch, '_RT')] = rt
-    arrays[add_to_bytestring(jets_branch, '_RTnew')] = rtnew
-    
+
+# Alternative RT values    
+def calculate_other_rts(arrays, jets_branch=b'JetsAK15_leading'):
+    jets = Jets(jets_branch, arrays, has_subjets=False, has_substructure=False)
+    mt = arrays[add_to_bytestring(jets_branch, '_MT')]
+    arrays[add_to_bytestring(jets_branch, '_RTnew')] = arrays[b'MET'] / np.sqrt(mt**2 + jets.pt**2)
+    arrays[add_to_bytestring(jets_branch, '_RT1')] = np.sqrt(1 + arrays[b'MET'] / jets.pt)
+
+def calculate_rt2(arrays, leading_branch=b'JetsAK15_leading', subleading_branch=b'JetsAK15_subleading'):
+    """
+    Only works assuming events with njets<=1 are filtered out
+    """
+    leading_jets = Jets(b'JetsAK15_leading', arrays, has_subjets=False, has_substructure=False)
+    subleading_jets = Jets(b'JetsAK15_subleading', arrays, has_subjets=False, has_substructure=False)
+    arrays[b'JetsAK15_RT2'] = np.sqrt(leading_jets.pt / subleading_jets.pt)
+
+
 def calculate_deltaeta(arrays, leading_branch=b'JetsAK15_leading', subleading_branch=b'JetsAK15_subleading'):
     leading_jets = Jets(leading_branch, arrays, has_subjets=False, has_substructure=False)
     subleading_jets = Jets(subleading_branch, arrays, has_subjets=False, has_substructure=False)
@@ -457,3 +507,15 @@ def calculate_dphimet(arrays, jets_branch=b'JetsAK15_subleading'):
     dphi[dphi > 2.*pi] = dphi[dphi > 2.*pi] - 2.*pi  # Whole circles subtracted
     dphi[dphi > pi] = 2.*pi - dphi[dphi > pi]  # Pick the smaller angle
     return dphi
+
+def normalize_dphi(dphi):
+    """
+    Normalizes a list of raw phi1-phi2 values: Subtracts 
+    whole 2*pi's, and then flips abs(values) > pi
+    """
+    dphi[dphi > 2.*pi] = dphi[dphi > 2.*pi] - 2.*pi
+    dphi[dphi < -2.*pi] = dphi[dphi < -2.*pi] + 2.*pi
+    # Then flip to -pi < phi < pi regime
+    dphi[dphi > pi] = dphi[dphi > pi] - 2.*pi
+    dphi[dphi < -pi] = dphi[dphi < -pi] + 2.*pi
+    return dphi
